#!@BIN_PERL@

# nagios: -epn
# disable embedded perl, works only under 3.x
# to avoid not preserving NAGIOS_* environment variables
#

#
#    Short description: Creates/Updates RRA archives, from nagios performance data
#    Copyright (C) 2006 - 2009  Badri Prasad Pillai
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

#
#    Contact details:
#		Badri(at)DigLinks.COM
#

use strict;

use RRDs;
use Getopt::Std;

# forward declaration
sub log_msg($);
sub load_config($);
sub usage();
sub list_default_values();
sub parse_perfdata($);
sub gen_rra_template();
sub gen_rra_string($$$$$);
sub get_check_interval($$$$);
sub load_plugin_map ($$);
sub create_dyn_rra_template ();

use vars qw(
  $version
  $login
  $opt
  $rrderror
  %config
  @parse_service_str
  $service
  @data
  $ds_name
  $ds_value
  %ds_rewrite
  $rrd_file
  $nagios_env_file
  $perf_rrd_file
  $fact_bytes
  $fact_seconds
  $perfdata
  $object
  $object_start
  $object_end
  $match_found
  $nagios_version
  $rra_string
  $plugin_maps_file
  $start_time
);

#
# Store current time
$start_time = time;

$version = "@VERSION@";
$login   = ( getpwuid($<) )[0];

$opt = {};

#
# store perfromance data label and values
$perfdata = {};

$rrderror = "";
my $object = {};

$object_start = undef;
$object_end   = undef;
$match_found  = 0;

$nagios_version = undef;

$rra_string = "";

#
# Do not buffer writes
$|++;

getopts( "c:C:dD:e:l:LN:hH:G:T:o:s:", $opt );

#
# If no configuration file given, then use default values
#
%config = (
   CONF_DIR                     => "@CONF_DIR@",
   TEMPLATES_DIR                => "@TEMPLATES_DIR@",
   SERVICE_NAME_MAPS            => "@TEMPLATES_DIR@/maps/service_name_maps",
   ZOOM_JS                      => "js/zoom.js",
   RRA_DIR                      => "@RRA_DIR@",
   LOGFILE                      => "@LOGFILE@",
   DOCUMENT_ROOT                => "@DOCUMENT_ROOT@",
   CACHE_DIR                    => "@CACHE_DIR@",
   RRDTOOL                      => "@RRDTOOL@",
   RRD_PATH_HIDDEN              => 1,
   NAGIOS_HOST_URL              => 1,
   THUMB_WIDTH                  => 200,
   THUMB_HEIGHT                 => 100,
   THUMB_DISPLAY                => "Daily",
   THUMB_DISPLAY_COLUMNS        => 3,
   SAVE_NAGIOS_ENVIRONMENT_VARS => 0,
   TMPDIR                       => "@TMPDIR@",
   CGIBIN                       => "@CGIBIN@",
   NAGIOS_CGIBIN                => "nagios/cgi-bin",
   TEMPLATE_NAME_AT_BEGINNING   => 0,
   DEMO_STR_LENGTH              => 3,
   PREDICTION_Daily             => "+2d",
   PREDICTION_Weekly            => "+7d",
   PREDICTION_Monthly           => "+15d",
   PREDICTION_Yearly            => "+6m",
   DYN_RRA_CREATE               => 0,
   DYN_GRAPH_CREATE             => 0,
   DYN_ADD_NEW_DS               => 0,
   DYN_RRA_PLUGIN_MAPS          => "@TEMPLATES_DIR@/maps/rra_plugin_maps",
   DYN_RRA_XFF                  => 0.5,
   DYN_CF                       => "AVERAGE,MAX,MIN,LAST",
   DYN_ARCHIVE_STEPS_DAYS       => "1:2,30:32,120:189,1440:1890",
   DYN_DS_CONVERT_TO_BYTES      => 0,
   DYN_DS_CONVERT_TO_SECONDS    => 0,
   DYN_RGB_COLORS_MAPS          => "@TEMPLATES_DIR@/maps/rgb.txt",
   DYN_GRAPH_USED_VALUE_COLOR   => "33a3ff",
   NAGIOS_STATUS_FILE           => "@NAGIOS_STATUS_FILE@",
   OPTIONAL_IMAGE_FORMATS       => 0,
   OPTIONAL_CGIBIN2             => "cgi-bin",

);

# declare convertion values
$fact_bytes = {
   "KB" => {
      'val'    => 2**10,    #KiloBytes
      'result' => "B"
   },
   "MB" => {
      'val'    => 2**20,    #MegaBytes
      'result' => "B"
   },
   "GB" => {
      'val'    => 2**30,    #GigaBytes
      'result' => "B"
   },
   "TB" => {
      'val'    => 2**40,    #TeraBytes
      'result' => "B"
   },
   "PB" => {
      'val'    => 2**50,    #PetaBytes
      'result' => "B"
   },
   "EB" => {
      'val'    => 2**60,    #ExaBytes
      'result' => "B"
   },
};

$fact_seconds = {
   "MS" => {
      'val'    => 10**-3,    #millisecond
      'result' => 's'
   },
   "US" => {
      'val'    => 10**-6,    #microsecond
      'result' => 's'
   },
   "NS" => {
      'val'    => 10**-9,    #nanosecond
      'result' => 's'
   }
};

#log_msg "startup args: %opt\n" if $opt->{d};

list_default_values if $opt->{L};
usage               if $opt->{h};

load_config( $opt->{c} ) if $opt->{c};

$plugin_maps_file = "$config{CONF_DIR}/$config{DYN_RRA_PLUGIN_MAPS}";

#
# Sanity check: if RRA directory exists
log_msg "Direcotry rra_dir = \"$config{CONF_DIR}\" doesn't exists\n"
  and exit 1
  if !-d $config{CONF_DIR};

#
# Sanity check: if template directory exists
log_msg "Directory templates_dir = \"$config{TEMPLATES_DIR}\" doesn't exists\n"
  and exit 1
  if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}";

#
# All three arguments are required
#
usage if !$opt->{H};
usage if !$opt->{s};
usage if !$opt->{o};

# only if dynamic creation of RRA is enabled
if ( $config{DYN_RRA_CREATE} ) {
   usage if !$opt->{N};
   usage if !$opt->{C};
}

if ( $opt->{D} ) {
   $opt->{D} = "SERVICE" if $opt->{D} !~ /(HOST|SERVICE)/;
}
else {
   $opt->{D} = "SERVICE";
}

if ( $opt->{d} ) {
   my $s = "";
   if ( $opt->{G} ) {
      $s =
"Hostgroup = $opt->{G}, Host = $opt->{H}, Service name = $opt->{s}, Check result = $opt->{o}\n";
   }
   else {
      $s =
"Host = $opt->{H}, Service name = $opt->{s}, Check result = $opt->{o}\n";
   }
   log_msg $s;
}

$service = "";

#
# If no map file exist, then silently ignore
#
if ( -f "$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}" ) {

# if you use _ in your service name and you don't want cut the name, you can simulate a rewrite
   open S_MAPS, '<', "$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}"
     or log_msg
"Can't open service maps file \"$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}\"\n"
     and exit 1;
   log_msg
"..Searching map in file \"$config{CONF_DIR}/$config{SERVICE_NAME_MAPS}\" for service \"$opt->{s}\" \n"
     if $opt->{d};

   while (<S_MAPS>) {
      next if /^#/;    # Skip comments
      next if /^$/;    # Skip empty lines
      s/#.*//;         # Remove partial comments
      chomp;

      if (/^$opt->{s}:\s*(\S+)/i) {
	 $service = $1 if /^$opt->{s}:\s*(\S+)/i;
	 log_msg "...Mapping service $opt->{s} -> $service\n" if $opt->{d};
      }
   }
   close S_MAPS;
}

if ( !$service ) {

   # NO Mapping Found

   # assume delimiter between service name is '_'
   @parse_service_str = split /_/, $opt->{s};

   if ( $config{TEMPLATE_NAME_AT_BEGINNING} ) {

      # The first word after terminator character "_"
      $service = $parse_service_str[0];
   }
   else {

      # The last word after terminator character "_"
      $service = $parse_service_str[$#parse_service_str];
   }
}

#
# assume delimiter between ds_name and ds_value is '='

@data = ();

# this will also remove spaces and other funny characters
#$service =~ s/[^\w]/_/g;# this will also remove spaces and other funny characters

# possibility for external performance data parsing
# First check for template with hostname_service
my $code_file =
     "$config{CONF_DIR}/$config{TEMPLATES_DIR}/code/$opt->{H}_${service}.pl" ;

if ( !-f $code_file ) {

      #
      # Else look for default template with $service.pl
      $code_file =
        "$config{CONF_DIR}/$config{TEMPLATES_DIR}/code/${service}.pl" ;

      if ( !-f $code_file ) {
	    log_msg
            "No external perl code found for host \"$opt->{H}\" and service \"${service}\"\n" 
            if $opt->{d};
       }
}

if ( -f $code_file ) {
   open P, '<', "$code_file"
     or die
"Can't open perl code file \"$code_file\"";

   my @PCODE = <P>;
   close P;

   log_msg
"Running perl code file $code_file for host \"$opt->{H}\" and service \"$opt->{s} ($service)\"\n"
     if $opt->{d};

   #
   # For external code uses nagios variables to get the value
   my $ret_str = eval join( "\n", @PCODE );
   warn $@ if $@;

   log_msg
"Perl code result for host \"$opt->{H}\" and service \"$service\" returned string \"$ret_str\"\n"
     if $opt->{d};

   #
   # remove spaces before and/or after "=" character
   $ret_str =~ s/\s+=/=/g;
   $ret_str =~ s/=\s+/=/g;

   @data = parse_perfdata($ret_str);
}
else {

   #
   # remove spaces before and/or after "=" character
   $opt->{o} =~ s/\s+=/=/g;
   $opt->{o} =~ s/=\s+/=/g;
   @data = parse_perfdata( $opt->{o} );
}

log_msg "Missing ds_name in Output: $opt->{o}, for service $opt->{s}\n"
  and exit 1
  if !@data;
log_msg "@data\n" if $opt->{d};

$ds_name  = "";
$ds_value = "";

foreach my $kv (@data) {
   my ( $key, $val ) = split /=/, $kv;
   $ds_name .= "$key:" if $key;
   $ds_value .= "$val:";
}
$ds_name  =~ s/:$//;
$ds_value =~ s/:$//;

%ds_rewrite = ();

# basedir is HOSTGROUPNAME
if ( $opt->{G} ) {
   $rrd_file      = "$config{RRA_DIR}/$opt->{G}/$opt->{H}_${service}.rrd";
   $perf_rrd_file = "$config{RRA_DIR}/$opt->{G}/$opt->{H}_${service}_el.rrd";

   #
   # create dir if it doesn't exist
   mkdir "$config{RRA_DIR}/$opt->{G}" if !-d "$config{RRA_DIR}/$opt->{G}";
}
else {
   $rrd_file      = "$config{RRA_DIR}/$opt->{H}/$opt->{H}_${service}.rrd";
   $perf_rrd_file = "$config{RRA_DIR}/$opt->{H}/$opt->{H}_${service}_el.rrd";

   #
   # create dir if it doesn't exist
   mkdir "$config{RRA_DIR}/$opt->{H}" if !-d "$config{RRA_DIR}/$opt->{H}";
}

#
# Check if a rewrite rules exist
#
if (
   -f "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rewrite/$opt->{H}_${service}_rewrite"
  )
{

   #
   #	print "Rewrite file detected for: $opt->{H}_${service}\n";
   open REWRITE, '<',
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/rewrite/$opt->{H}_${service}_rewrite"
     or log_msg "Can't open rewrite rules file for $opt->{H}\n" and exit 1;

   my $ds_name_tmp = "";

   while (<REWRITE>) {
      next if /^#/;    # Skip comments
      next if /^$/;    # Skip empty lines
      s/#.*//;         # Remove partial comments
      chomp;

      $rrd_file      = $1 if /^rrd_file\s+(\S+)/;
      $perf_rrd_file = $1 if /^perf_rrd_file\s+(\S+)/;
      if (/^ds_name\s+([\S\\\/ ]+)\s+(\w+)/o) {
	 $ds_name_tmp .= "$2:";
	 $ds_rewrite{$1} = $2;

	 # better change ref content too, assume the ds_name is RRDTool conform
	 $perfdata->{$1}->{clean} = $2 if defined $perfdata->{$1}->{clean};
      }
   }
   close REWRITE;

   #print "rrd_file = $rrd_file\n";
   if ($ds_name_tmp) {
      $ds_name_tmp =~ s/:$//;
      $ds_name = $ds_name_tmp;
   }
}

$nagios_env_file = $rrd_file;
$nagios_env_file =~ s/\.rrd//;
$nagios_env_file .= ".env";

#
# if SAVE_NAGIOS_ENVIRONMENT_VARS is disabled
# remove env files if they exists
#
unlink $nagios_env_file if -f $nagios_env_file;

if ( $config{SAVE_NAGIOS_ENVIRONMENT_VARS} ) {

   # save NAGIOS environemnt values
   open E, '>', "$nagios_env_file"
     or log_msg "can't open file \"$nagios_env_file\"\n" and exit 1;
   log_msg "Writing ENV variables to file $nagios_env_file\n" if $opt->{d};
   foreach my $e ( keys %ENV ) {
      next if $e !~ /^NAGIOS/;
      print E "${e}=${ENV{$e}}\n" if length( $ENV{$e} ) > 1;
   }
   close E;
}

#
# Create/Update
#
if ( -e $rrd_file ) {

   #
   # If not time given
   my $time = $opt->{T} || "N";

   log_msg
     "Updating ds_names: $ds_name, ds_values: $ds_value in file: $rrd_file\n"
     if $opt->{d};

   RRDs::update( "$rrd_file", "--template", "$ds_name", "$time:$ds_value");
   $rrderror = RRDs::error;
   if (  $rrderror
      && ( $rrderror =~ /(unknown\s+DS|more DS definitions than RRD)/ )
      && $config{DYN_ADD_NEW_DS}
      && $config{DYN_RRA_CREATE} )
   {
      eval { use RRD::Simple (); };

      if ($@) {
	 log_msg "Perl module RRD::Simple not installed\n";
	 exit 1;
      }

# Now create the template which fills up perfdata ref with some values needed later
      my $rrd_create_template = create_dyn_rra_template;

      my $rrd = RRD::Simple->new(
	 file    => "$rrd_file",
	 rrdtool => $config{RRDTOOL},
      );

      # extract DS names from RRD File
      my %dsnames = map { $_, 1 } $rrd->sources;

      # store as hash the ds names, makes easier for checking
      my %dsnames_current = map { $_, 1 } split /:/, $ds_name;

      my @ds_new = ();

      foreach my $chk ( keys %dsnames_current ) {
	 push @ds_new, $chk if not defined $dsnames{$chk};
      }

      if (@ds_new) {

	 #
	 # add new DS one by one
	 foreach my $ds (@ds_new) {
	    log_msg "Adding ds \"$ds\" to \"$rrd_file\"\n" if $opt->{d};

	    # guess DS TYPE
	    my $ds_type = "GAUGE";
	    foreach my $l ( keys %{$perfdata} ) {
	       $ds_type = $perfdata->{$l}->{ds} and last
		 if $perfdata->{$l}->{clean} = "$ds" and $perfdata->{$l}->{ds};
	    }
	    $rrd->add_source( $rrd_file, $ds => $ds_type );
	 }
	 RRDs::update( "$rrd_file", "--template", "$ds_name",
	    "$time:$ds_value" );
	 $rrderror = RRDs::error;
	 log_msg "ERROR while updating $rrd_file: $rrderror\n" and exit 1
	   if $rrderror;
      }
      else {

# If no new DS, then something else is wrong, so print the last rrderror and exit
	 log_msg "ERROR while updating $rrd_file: $rrderror\n" and exit 1;
      }
   }
   elsif ($rrderror) {
      log_msg "ERROR while updating $rrd_file: $rrderror\n" and exit 1;
   }
}
else {

   #
   # First check for template with hostname
   my $rrd_create_template =
     "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/$opt->{H}_${service}.t";

   if ( !-f $rrd_create_template ) {

      #
      # Else look for default template
      $rrd_create_template =
	"$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/${service}.t";

      if ( !-f $rrd_create_template ) {
	 if ( $config{DYN_RRA_CREATE} ) {
	    $rrd_create_template = create_dyn_rra_template;
	 }
	 else {
	    log_msg
"Missing template for \"$opt->{H}\" service \"$opt->{s}\" \"$rrd_create_template\"\n";
	    exit 1;
	 }
      }
   }

   log_msg "Template for service \"${service}\" -> \"$rrd_create_template\"\n"
     if $opt->{d};

   open T, '<', "$rrd_create_template"
     or log_msg "can't open file \"$rrd_create_template\"\n" and exit 1;

   my @t_params = ();

   while (<T>) {
      next if /^#/;    # Skip comments
      next if /^$/;    # Skip empty lines
      s/#.*//;         # Remove partial comments

      chomp;

      #
      # if there are ds_name rewrites needed
      foreach my $k ( keys %ds_rewrite ) {
	 s/:$k:/:$ds_rewrite{$k}:/;
      }

      push @t_params, "$_";
   }

   close T;

   log_msg "Creating RRD file: $rrd_file\n" if $opt->{d};
   RRDs::create( $rrd_file, @t_params );
   $rrderror = RRDs::error;
   log_msg "Problem while creating rrd: $rrderror\n" and exit 1 if ($rrderror);
}

#
# Store execution and latency performance data
if ( $opt->{e} and $opt->{l} ) {
   my $time = $opt->{T} || "N";
   $opt->{e} ||= 0;
   $opt->{l} ||= 0;

   my $s =
"Host = $opt->{H}, Service name = $opt->{s}, Execution time = $opt->{e}, Latency = $opt->{l}\n";
   log_msg $s if $opt->{d};

   if ( -e $perf_rrd_file ) {
      log_msg
	"Updating execution and latency performance in file: $perf_rrd_file\n"
	if $opt->{d};
      RRDs::update(
	 "$perf_rrd_file",    "--template",
	 "execution:latency", "$time:$opt->{e}:$opt->{l}"
      );
      $rrderror = RRDs::error;
      log_msg
"ERROR while updating $perf_rrd_file: $rrderror e =$opt->{e} l = $opt->{l}\n"
	and exit 1
	if $rrderror;
   }
   else {

      #
      # First check for template with hostname and service name
      # it can be that a perticular service is polled at different interval than
      # default 500 seconds
      #
      my $perf_rrd_create_template =
	"$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/${service}_el.t";

      if ( !-f $perf_rrd_create_template ) {

	 #
	 # Else look for default template
	 $perf_rrd_create_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/default_execution_latency_el.t";
	 if ( !-f $perf_rrd_create_template ) {
	    log_msg
"Missing execution and latency template \"$perf_rrd_create_template\"\n";
	    exit 1;
	 }
      }

      open EL, '<', "$perf_rrd_create_template"
	or log_msg "can't open file \"$perf_rrd_create_template\"\n" and exit 1;

      my @el_params = ();

      while (<EL>) {
	 next if /^#/;    # Skip comments
	 next if /^$/;    # Skip empty lines
	 s/#.*//;         # Remove partial comments

	 chomp;

	 push @el_params, "$_";
      }
      close EL;

      log_msg
	"Creating RRD execution and latency performance file: $perf_rrd_file\n"
	if $opt->{d};
      RRDs::create( $perf_rrd_file, @el_params );
      $rrderror = RRDs::error;
      log_msg "Problem while creating execution and latency rrd: $rrderror\n"
	and exit 1
	if ($rrderror);
   }

}

#
#

# Execution time
log_msg(
   "Total running time($service): " . ( time - $start_time ) % 60 . "s\n" )
  if $opt->{d};

exit 0;

sub log_msg ($) {
   my $str = shift;

   my ($line) = ( caller(0) )[2];

   open L, '>>', "$config{LOGFILE}"
     or die "called in line $line: Can't open logfile \"$config{LOGFILE}\"";

   open STDERR, '>&L';    # dup STDERR

   print L scalar localtime() . ":n2rrd $line $login:$opt->{H}: $str";

   close L;
}

sub load_config ($) {
   my $config_file = shift;
   my $line        = 0;

   open CONF, '<', "$config_file"
     or die "Can't open configuration file -> \"$config_file\":$!\n";

   #
   # Parse configuration file
   while (<CONF>) {
      $line++;    # note confirguration file line number
      chomp;      # Remove newline character
      s/ //g;     # remove spaces
      next if /^#/;    # Skip comments
      next if /^$/;    # Skip empty lines
      s/#.*//;         # Remove partial comments

      if (/^(.*)=(.*)$/) {
	 if ( defined $config{$1} ) {
	    $config{$1} = $2;
	 }
	 else {        # A warning should be ok
	    log_msg
"WARNING: Unknown global variable declaration in line $line:\"$1 = $2\"\n";
	 }
      }
   }
   close CONF;
}

#
# funtion that parses performance data
# the result is stored in global ref $perfdata
sub parse_perfdata($) {
   my $perf_str = shift;

   # empty array
   $perfdata = {};

   my @tmp_data = ();
   my @vals     = (
      $perf_str =~ /([^=]+	  # var name
             =[-+]?[0-9\.,]+       # value mandatory
              [\w%]*[ ;]?         # uom
	      [-+]?[0-9\.,]*;?     # warning
              [-+]?[0-9\.,]*;?     # critical
	      [-+]?[0-9\.,]*;?     # min
	      [-+]?[0-9\.,]*;?\s*) # max
	   /xsmg
   );

   foreach my $v (@vals) {

      my ( $label, $val, $uom, $w, $c, $min, $max ) = (
	 $v =~ /([^=]+)	           # var name
                     =([-+]?[0-9\.,]+)      # value mandatory
                      ([\w%]*)[ ;]?        # uom
                      ([-+]?[0-9\.,]*);?    # warning
                      ([-+]?[0-9\.,]*);?    # critical
                      ([-+]?[0-9\.,]*);?    # min
                      ([-+]?[0-9\.,]*);?\s* # max
                   /xg
      );

      $uom = $uom || "NONE";
      $uom = uc $uom;

      $w   = $w   || 0;
      $c   = $c   || 0;
      $min = $min || 0;
      $max = $max || 0;

  #
  # I assume only "," or "." are part of numbers
  # in my case I found that if system LANG is set to "DE*", the output format of
  # some plugins are affected, which in turn breakes rrd data update
  #
      $val =~ s/,/./g if $val;
      $w   =~ s/,/./g if $w;
      $c   =~ s/,/./g if $c;
      $min =~ s/,/./g if $min;
      $max =~ s/,/./g if $max;

      $perfdata->{$label}->{value}    = $val;
      $perfdata->{$label}->{uom}      = $uom;
      $perfdata->{$label}->{warning}  = $w;
      $perfdata->{$label}->{critical} = $c;
      $perfdata->{$label}->{min}      = $min;
      $perfdata->{$label}->{max}      = $max;

# Convert only if enabled
# value, warning, critical, min, max are all UOM based values as per Nagios documents
      my @values = qw(
	value
	warning
	critical
	min
	max
      );

      if ( defined $fact_bytes->{ $perfdata->{$label}->{uom} }
	 and $config{DYN_DS_CONVERT_TO_BYTES} )
      {
	 my @t = map {
	    $perfdata->{$label}->{$_} =
	      $perfdata->{$label}->{$_} *
	      $fact_bytes->{ $perfdata->{$label}->{uom} }->{val}
	 } @values;
	 $perfdata->{$label}->{uom} =
	   $fact_bytes->{ $perfdata->{$label}->{uom} }->{result};
      }
      elsif ( defined $fact_seconds->{ $perfdata->{$label}->{uom} }
	 and $config{DYN_DS_CONVERT_TO_SECONDS} )
      {
	 my @t = map {
	    $perfdata->{$label}->{$_} =
	      $perfdata->{$label}->{$_} *
	      $fact_seconds->{ $perfdata->{$label}->{uom} }->{val}
	 } @values;
	 $perfdata->{$label}->{uom} =
	   $fact_seconds->{ $perfdata->{$label}->{uom} }->{result};
      }

      if ( $label eq '/' ) {
	 $perfdata->{$label}->{clean} = "root" if $label eq '/';
      }
      else {
	 my $temp_label = $label;

	 #
	 # rrd DS names are limited to 19 characters
	 # and only alphanumeric plus "_" is allowed
	 $temp_label =~ s/'//g;# remove
	 $temp_label =~ s/[^\w]/_/g;
	 $temp_label = substr( $temp_label, 0, 19 )
	   if length($temp_label) > 19;

	 $perfdata->{$label}->{clean} = $temp_label;
      }
      push @tmp_data,
	"$perfdata->{$label}->{clean}=$perfdata->{$label}->{value}";
   }
   return @tmp_data;
}

#
# This function generates RRA template and stores the result in $rra_string;
#
sub gen_rra_template() {

   my $service_check_interval =
     get_check_interval( "$opt->{H}", "$opt->{D}", "$opt->{s}", "$opt->{N}" );
   $service_check_interval = 5 if $service_check_interval <= 0;

   # step and heartbeat
   my $s         = $service_check_interval * 60;
   my $heartbeat = $s * 2;

   # reset global variable rrd_string
   $rra_string = "";

   my @DYN_CF                 = split /,/, $config{DYN_CF};
   my @DYN_ARCHIVE_STEPS_DAYS = split /,/, $config{DYN_ARCHIVE_STEPS_DAYS};

   # generate DS info
   $rra_string =
       "#Dynamically Generated RRA template\n#"
     . scalar localtime()
     . "\n#RRD_FILENAME = $rrd_file\n#\n";
   $rra_string .= "-s $s\n";

   # Generate DS lines
   foreach my $l ( keys %{$perfdata} ) {
      $rra_string .= "#Original label = " . "$l\n";
      if ( $perfdata->{$l}->{uom} eq '%' ) {    # percent
	 $rra_string .= "DS:$perfdata->{$l}->{clean}:GAUGE:$heartbeat:U:U\n";
      }
      elsif ( $perfdata->{$l}->{uom} eq 'C' ) {    # assume counter
	 $rra_string .= "DS:$perfdata->{$l}->{clean}:COUNTER:$heartbeat:0:U\n";
      }
      else {
	 $rra_string .=
	     "DS:$perfdata->{$l}->{clean}:"
	   . ( $perfdata->{$l}->{ds} ? "$perfdata->{$l}->{ds}" : "GAUGE" )
	   . ":$heartbeat:"
	   . ( $perfdata->{$l}->{min} ? "$perfdata->{$l}->{min}:"  : "U:" )
	   . ( $perfdata->{$l}->{max} ? "$perfdata->{$l}->{max}\n" : "U\n" );
      }
   }

   # Generate RRA lines
   foreach my $c (@DYN_CF) {
      $c =~ s/\W//g;
      log_msg("DEBUG: DYN_CF: $c\n") if $opt->{d};
      foreach my $a (@DYN_ARCHIVE_STEPS_DAYS) {
	 my ( $s, $d ) = split /:/, $a;
	 $d =~ s/\W//g;

	 my $rra_tmp_str =
	   gen_rra_string( $c, $config{DYN_RRA_XFF}, $s,
	    $service_check_interval, $d );
	 if ( $rra_tmp_str =~ /^ERROR/ ) {
	    log_msg "$rra_tmp_str\n";
	    exit 2;
	 }
	 else {
	    $rra_string .= $rra_tmp_str . "\n";
	 }
      }
   }
   return 0;
}

#
# return string of following format
# RRA:AVERAGE | MIN | MAX | LAST:xff:steps:rows
#
sub gen_rra_string($$$$$) {
   my ( $cf, $xff, $steps, $check_interval, $archive_period_in_days ) = @_;

   my $string = "";

   # sanity check CF
   if ( $cf =~ /^(AVERAGE|MIN|MAX|LAST)$/ ) {
      $string = "RRA:$cf:";
   }
   else {
      return "ERROR: unknown CF \"$cf\"";
   }

   # sanity check XFF

   if ( ( $xff < 0 ) || ( $xff > 1 ) ) {
      return "ERROR: invalid XFF value \"$xff\"";
   }
   else {
      $string .= "$xff:";
   }

   # sanity check STEPS, must be possitive integer
   $steps =~ s/\W//g;
   if ( $steps =~ /^\d+$/ ) {
      $string .= "$steps:";
   }
   else {
      return "ERROR: invalid value for steps \"$steps\"";
   }

   # sanity check archive_period
   if ( $archive_period_in_days =~ /^\d+$/ ) {
      my $total_seconds = 24 * $archive_period_in_days * 60 * 60;
      my $rows = int( $total_seconds / ( $check_interval * 60 * $steps ) );
      $string .= "$rows";
   }
   else {
      return
	"ERROR: invalid archive_period_in_days \"$archive_period_in_days\"";
   }

   return $string;
}

sub get_check_interval($$$$) {
   my ( $hostname, $o_type, $service_description, $status_file ) = @_;

   log_msg("DEBUG: $hostname, $o_type, $service_description, $status_file\n")
     if $opt->{d};
   open STATUS, "<$status_file"
     or die "can't open status file \"$status_file\"";
   while ( my $line = <STATUS> ) {
      if ( $line =~ /\s*(info|service|servicestatus|host|hoststatus)\s*{.*$/ ) {
	 $object_start = $1;
      }
      elsif ( $object_start && $line =~ /\s*(\w+)=(.*)$/ ) {
	 $object->{$object_start}->{$1} = "$2";
      }
      elsif ( $object_start && $line =~ /\s*}$/ ) {
	 $object_end++;
      }
      else { next; }

      #
      # parsed one object and its value
      if ($object_end) {
	 if ( $object_start eq 'info' ) {
	    $nagios_version = $object->{$object_start}->{version}
	      if $object_start =~ /info/;
	    $nagios_version =~ s/(\d+).*/$1/;
	    return 0 if $nagios_version < 2;

	    next;
	 }

	 if (  ( defined $object->{$object_start}->{host_name} )
	    && ( $object->{$object_start}->{host_name} =~ /$hostname/ ) )
	 {
	    my $check_interval = 0;
	    $check_interval = int(
	       (
		  $object->{$object_start}->{next_check} -
		    $object->{$object_start}->{last_check}
	       ) / 60
	    ) if $nagios_version == 2;

	    if ( $nagios_version == 3 )
	    { # between 3.0 and 3.1 variable names are different, but the values are from same source
	       $check_interval =
		 int( $object->{$object_start}->{check_interval} )
		 if defined $object->{$object_start}->{check_interval};
	       $check_interval =
		 int( $object->{$object_start}->{normal_check_interval} )
		 if defined $object->{$object_start}->{normal_check_interval};
	    }

	    if ( $object_start =~ /hoststatus/ && $o_type =~ /host/i ) {
	       return "$check_interval";
	    }
	    elsif ( $object_start =~ /service/ ) {
	       $match_found++
		 if $object->{$object_start}->{service_description} =~
		    /$service_description/
		    and $object->{$object_start}->{host_name} =~ /$hostname/;
	       return "$check_interval" if $match_found;
	    }
	 }
      }

      if ($object_end) {
	 $object_start = undef;
	 $object_end   = undef;
	 $match_found  = 0;
	 $object       = {};
      }
   }
   return 0;
}

#
# Create object(plugin name) with attributes(labels) and DS name
#
sub load_plugin_map ($$) {
   my ( $filename, $plugin_name ) = @_;

   # return if for some reason var not set
   return 0 if not $filename;
   return 0 if not $plugin_name;

   open PMAPS, "<$filename" or die "Can't open plugin maps file \"$filename\"";
   while (<PMAPS>) {
      next if !/^$plugin_name/;
      chomp;
      my ( $plugin, $labels ) = split /=/, $_, 2;
      my @labeldef = split /,/, $labels;
      foreach my $l (@labeldef) {
	 my ( $l, $d ) = split /:/, $l;
	 $perfdata->{$l}->{ds} = $d if $d =~ /(GAUGE|COUNTER|DERIVE|ABSOLUTE)/;
      }
      last;    # as we don't look for more than one entry
   }
   close PMAPS;
}

sub create_dyn_rra_template () {

   # create dny directory, if not there
   mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/dyn"
     if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/dyn";

   # create host directory, if not there
   mkdir "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/dyn/$opt->{H}"
     if !-d "$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/dyn/$opt->{H}";

   my $rrd_create_template =
"$config{CONF_DIR}/$config{TEMPLATES_DIR}/rra/dyn/$opt->{H}/$opt->{H}_${service}.t";

   #
   # Recreate the template
   #
   unlink $rrd_create_template if -f $rrd_create_template;

   log_msg "Creating RRA template \"$rrd_create_template\" dynamically\n";

   #
   # if plugins maps exist load it
   load_plugin_map( "$plugin_maps_file", "$opt->{C}" ) if -f $plugin_maps_file;

   gen_rra_template;
   open DYN_TEMPLATE, '>', "$rrd_create_template"
     or log_msg "can't create file \"$rrd_create_template\"\n" and exit 1;
   print DYN_TEMPLATE "$rra_string";
   close DYN_TEMPLATE;

   return $rrd_create_template;
}

sub usage () {
   my ($line) = ( caller(0) )[2];

   print "called at line: $line
NOTE: nagios 1|2|3.x variables may differ
n2rrd: $version\n n2rrd options
	-h 		print usage and exit
	-c <path>/config-file-name 
			n2rrd.conf in case you want to overide default values
	-C check_command 
			\"\$HOSTCHECKCOMMAND\$|\$SERVICECHECKCOMMAND\$\"
	-d		debug
	-D		input performance data is for HOST|SERVICE check (DEFAULT is SERVICE)
	-L 		this will print the default configuration values and exit
	-N <patch>/status-file
			nagios \$STATUSDATAFILE\$ used for dynamic RRA creation
			
	-e Elapsed_time \$SERVICEEXECUTIONTIME\$
	-l Latence_time \$SERVICELATENCY\$
	-T time 	\$LASTHOSTCHECK\$
	-H hostname 	\$HOSTNAME\$ 
	-G hostgroup    \$HOSTGROUPNAME\$
	-s servicename  \"\$SERVICEDESC\$\"
	-o service_check_output
			\"\$SERVICEPERFDATA\$\"\n\n";
   exit 0;
}

sub list_default_values() {
   load_config( $opt->{c} ) if $opt->{c};

   printf "#\n#n2rrd: $version\n";
   printf "#%24s = %s\n", "Key  ", "  Value";
   printf "%25s = %s\n", "#" x 25, "#" x 30;
   foreach my $k ( sort keys %config ) {
      printf "%25s = %s\n", $k, $config{$k};
   }
   exit 0;
}
