--lazy
--title="$HOSTNAME$ - ($SERVICENAME$) Usage"
--base=1024
--height=200
--width=500
--alt-autoscale-max
--lower-limit=0
--vertical-label=Bytes
--step=500

# 
# The filesystem dataset 
# 
DEF:a=$RRD_FILENAME$:used:AVERAGE
DEF:b=$RRD_FILENAME$:free:AVERAGE
DEF:u_days=$RRD_FILENAME$:used:AVERAGE:step=500:start=now-3d 
DEF:f_days=$RRD_FILENAME$:free:AVERAGE:step=500:start=now-3d 
DEF:u_week=$RRD_FILENAME$:used:AVERAGE:step=500:start=now-1w 
DEF:f_week=$RRD_FILENAME$:free:AVERAGE:step=500:start=now-1w 
DEF:u_month=$RRD_FILENAME$:used:AVERAGE:step=500:start=now-1m 
DEF:f_month=$RRD_FILENAME$:free:AVERAGE:step=500:start=now-1m 

#
# Current value
#
CDEF:current_used=a,1024,*
CDEF:current_free=b,1024,*
CDEF:current_total=current_used,current_free,+
CDEF:current_one_p=current_total,100,/
CDEF:current_p_used=current_used,current_one_p,/
CDEF:current_p_free=100,current_p_used,-
CDEF:current_p_80=current_one_p,80,*
CDEF:current_p_90=current_one_p,90,*
CDEF:current_p_99=current_one_p,99,*

#
# Days,Week,Month value K->Bytes
#
CDEF:days_used=u_days,1024,*
CDEF:week_used=u_week,1024,*
CDEF:month_used=u_month,1024,*

# 
# Calculate Least Squares Line (LSL) values for Current, Days, Week and Month
# 
VDEF:slope_current=current_used,LSLSLOPE 
VDEF:yint_current=current_used,LSLINT 
VDEF:correl_current=current_used,LSLCORREL 

VDEF:slope_days=days_used,LSLSLOPE 
VDEF:yint_days=days_used,LSLINT 
VDEF:correl_days=days_used,LSLCORREL 

VDEF:slope_week=week_used,LSLSLOPE 
VDEF:yint_week=week_used,LSLINT 
VDEF:correl_week=week_used,LSLCORREL 

VDEF:slope_month=month_used,LSLSLOPE 
VDEF:yint_month=month_used,LSLINT 
VDEF:correl_month=month_used,LSLCORREL 

# 
# Generate projection data
# 
CDEF:projuse_current=current_used,POP,yint_current,slope_current,COUNT,*,+ 
CDEF:fyline_current=projuse_current,0,current_total,LIMIT

CDEF:projuse_days=days_used,POP,yint_days,slope_days,COUNT,*,+ 
CDEF:fyline_days=projuse_days,0,current_total,LIMIT

CDEF:projuse_week=week_used,POP,yint_week,slope_week,COUNT,*,+ 
CDEF:fyline_week=projuse_week,0,current_total,LIMIT

CDEF:projuse_month=month_used,POP,yint_month,slope_month,COUNT,*,+ 
CDEF:fyline_month=projuse_month,0,current_total,LIMIT

# 
# Get the last value off of the projected data set. 
# 
VDEF:firstv=current_used,FIRST 
VDEF:lastv=projuse_current,LAST 

# 
# crossover limit is set to 99% of total disk space
# 
CDEF:crosslimit_current=projuse_current,current_p_99,INF,LIMIT,UN,UNKN,slope_current,0,LT,UNKN,TIME,IF,IF 
CDEF:crosslimit_days=projuse_days,current_p_99,INF,LIMIT,UN,UNKN,slope_days,0,LT,UNKN,TIME,IF,IF 
CDEF:crosslimit_week=projuse_week,current_p_99,INF,LIMIT,UN,UNKN,slope_week,0,LT,UNKN,TIME,IF,IF 
CDEF:crosslimit_month=projuse_month,current_p_99,INF,LIMIT,UN,UNKN,slope_month,0,LT,UNKN,TIME,IF,IF 

# 
# Pull the first known value from the crosslimit data sets defined 
# above. This will be the first value that hit the 99% mark. 
# Use the :strftime argument to gprint to print the timestamp of 
# this value later. 
# 
VDEF:co_limit_current=crosslimit_current,FIRST 
VDEF:co_limit_days=crosslimit_days,FIRST 
VDEF:co_limit_week=crosslimit_week,FIRST 
VDEF:co_limit_month=crosslimit_month,FIRST 

GPRINT:firstv:"Period between %d/%b/%Y ":strftime 
GPRINT:lastv:" and %d/%b/%Y\c":strftime 
COMMENT:"\s"  


# 
# Draw the base area using our base color (slight transparent yellow). 
# 
AREA:current_used#FF0000:"Used\: "
GPRINT:current_used:LAST:"%.2lf %s"
GPRINT:current_p_used:LAST:"(%.0lf%%)"

#
# STACK free space on top of used for total
#
AREA:current_free#cdcdcd:"Free\: ":STACK
GPRINT:current_free:LAST:"%.2lf %s"
GPRINT:current_p_free:LAST:"(%.0lf%%)"

# 
# Then draw the gradient on top of the base (using green 
# but changing the transparency of each slice. 
# 

CDEF:g_used_cl2=current_used,0.98,*
AREA:g_used_cl2#FF0000
CDEF:g_used_cl10=current_used,0.90,*
AREA:g_used_cl10#FF0000
CDEF:g_used_cl15=current_used,0.85,*
AREA:g_used_cl15#F20000
CDEF:g_used_cl20=current_used,0.80,*
AREA:g_used_cl20#E50000
CDEF:g_used_cl25=current_used,0.75,*
AREA:g_used_cl25#D90000
CDEF:g_used_cl30=current_used,0.70,*
AREA:g_used_cl30#CC0000
CDEF:g_used_cl35=current_used,0.65,*
AREA:g_used_cl35#BF0000
CDEF:g_used_cl40=current_used,0.60,*
AREA:g_used_cl40#B20000
CDEF:g_used_cl45=current_used,0.55,*
AREA:g_used_cl45#A60000
CDEF:g_used_cl50=current_used,0.50,*
AREA:g_used_cl50#990000
CDEF:g_used_cl55=current_used,0.45,*
AREA:g_used_cl55#8C0000
CDEF:g_used_cl60=current_used,0.40,*
AREA:g_used_cl60#7F0000
CDEF:g_used_cl65=current_used,0.35,*
AREA:g_used_cl65#730000
CDEF:g_used_cl70=current_used,0.30,*
AREA:g_used_cl70#660000
CDEF:g_used_cl75=current_used,0.25,*
AREA:g_used_cl75#590000
CDEF:g_used_cl80=current_used,0.20,*
AREA:g_used_cl80#4C0000
CDEF:g_used_cl85=current_used,0.15,*
AREA:g_used_cl85#400000



# 
# Top it all off with a darker line for contrast. 
# 
LINE:current_used#116611  

#
# Percent 99 and 80 line
#
COMMENT:"Percent mark "
LINE:current_p_90#0019BE:"90"
LINE:current_p_80#157A01:"80\n"



# 
# Now our ledgends and other info 
# 
COMMENT:"Projection\t\t\t" 
COMMENT:"Co-Variance\n" 
# 
COMMENT:"--------------------\t" 
COMMENT:"-----------\n"
# 
LINE2:fyline_current#CD12CD:"From graph start\t" 
GPRINT:correl_current:" %6.2lf\n" 
# 
LINE2:fyline_days#C0C0FF:"From 3 days ago \t" 
GPRINT:correl_days:" %6.2lf\n" 
# 
LINE2:fyline_week#FFFF00:"From 1 week ago \t" 
GPRINT:correl_week:" %6.2lf\n"
# 
LINE2:fyline_month#FF8000:"From 1 month ago \t"
GPRINT:correl_month:" %6.2lf\n"
#
